/*						///  Module mto F3F  \\\

	Serge Delarbre 15.05.15

*/

#include <stdio.h>
#include <stdlib.h>
#include <float.h>
#include <string.h>

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <avr/wdt.h>
#include <avr/pgmspace.h>

#include "GiraneDS.h"

//signature ATmega328P = 0x1E 0x95 0x0F
//origin fuse : Ex= 0xFF, Hi= 0xD9, Lo= 0x62

/*
FUSES= 
	{
   .low = 0xF7,
   .high = 0xD9,
   .extended = 0xFF,		//no clk / 8, osc quartz ext full swing 
	};
*/

#define MODULE_NAME "WinDS F3F"
#define VERSION "V0.0"

volatile int VTimer= 0;								//counter down for tempo ms

//__ calculs conditions de vol
volatile unsigned int	TimeBase= 0;			//time base for measure frequency phase A of speed wind coder
volatile int				Angle;					//counter for measure angle wind with angle coder, phase A and B
volatile int				TmpBeep;					//tempo activation buzzer
volatile int Vitesses[MAXMOY];					//buffer for speed filtering
volatile int Directions[MAXMOY];					//buffer for angle filtering
volatile int pntMeasure= 0;						//pointer in buffer speed and angle

int IndParam= 0;										//index parameters selection 
bool ShortClick= OFF;

bool	ReqAMC;											//tempo traitement Conditions marginales
unsigned int	CntCondition;						//marginals condition counter

volatile int Vbat;									//measure battery voltage

//__communication IHM
int Mode= MEASURE;									//measure or config
volatile char IT_IHM= FREE;						//state IT IHM
char Dialog[SIZ_BUF_RX];
//___ Parameters saved in EEPROM
int CFG[MAXPARAM];



//__________ IT millisecond timer 2
ISR(TIMER2_OVF_vect)
{
TCNT2-= IT_1ms;

//___ base time second for angle & speed acquisitions
if(Mode == MEASURE)
	{
	if(TimeBase-- <= 0)
		{
		pntMeasure++;
		if(pntMeasure >= CFG[iFilteringAlarm]) pntMeasure= 0;
		//__phase speed coder frequency memo
		Vitesses[pntMeasure]= TCNT1;
		TCNT1= 0;
		//__angle memo 
		Directions[pntMeasure]= Angle;
		//__	
		ReqAMC= ON;											//request display actualisation and treatment marginals condition
	
		GetVbat();											//measure battery voltage

		TimeBase= SECOND;
		if(IT_IHM == FREE) IT_IHM= ANGCODER;
		}
	}

//___ gestion buzzer
if(TmpBeep > 0)
	TmpBeep--;
else
	BUZZER= OFF;

//___ gestion mean tempo 
if(VTimer > 0) VTimer--;
}

//__________ IT coder angle and IHM
int param, paramax, paramin;
ISR(PCINT1_vect)
{
if(Mode == MEASURE)
	{	//___ angle coder (double rsolution)	
	if(ANG_VA == 1)		//IT phase A rising edge 
		{
		if(ANG_VB == 0)
			Angle++;
		else
			Angle--;
		}
	else
		{						//IT phase A falling edge 
		if(ANG_VB == 1)
			Angle++;
		else
			Angle--;
		}
	}
else
	{	//config mode
	//___ phase A IHM coder
	if(IHM_A == 0)		//IT phase A falling edge 
		{
		if(IHM_B == 1)
			{		//clockwise
			if(ShortClick)
				{		//param increment
				CFG[param]++;
				Limit(&CFG[param], paramax, paramin);
				}
			else
				{		//next param
				IndParam++;
				if(IndParam >= MAXEDIT) IndParam= 0;		//looping
				}
			}
		else
			{		//counter clockwise
			if(ShortClick)
				{		//param decrement
				CFG[param]--;
				Limit(&CFG[param], paramax, paramin);
				}
			else
				{		//previous param
				if(IndParam <= 0) IndParam= MAXEDIT;		//looping
				IndParam--;
				}
			}

		IT_IHM= IHMCODER;
		}
	}
}

//__________ reading analog input (battery voltage), called all seconds : see millisecond IT
void GetVbat(void)
{
if((ADCSRA & (1 << ADSC)) == 0) 		
	{		//__conversion finish
	int l= ADCL;
	int h= ADCH;
	Vbat= h * 256 + l;
	}
	
ADCSRA|= (1 << ADSC);	//start next single conversion
}


//______________________________ Tempo gestion ______________________________

//__________ Arm tempo, specify in ms
void TimerSet(int t)
{
VTimer= t;
}

//__________ Vrai si tempo en cours
bool TimerActive(void)
{
return(VTimer > 0);
}


//______________________________ Parameters memory ______________________________

//__________ Saving parameters to EEPROM
void SaveCFG(void)
{
int i;
uint16_t *ad;
uint16_t ChekSum;

ChekSum= 0;
ad= 0;
for(i= 0; i < MAXPARAM; i++)
	{
	eeprom_write_word((uint16_t*)ad, (uint16_t)CFG[i]);
	ad+= sizeof(CFG[i]);
	ChekSum+= CFG[i];
	}

eeprom_write_word((uint16_t*)ad, (uint16_t)ChekSum);
}

//__________ Loadind parameters from EEPROM to RAM
void LoadCFG(void)
{
int i;
uint16_t *ad;
uint16_t ChekSum, cks;

ChekSum= 0;
ad= 0;
for(i= 0; i < MAXPARAM; i++)
	{
	CFG[i]= eeprom_read_word((uint16_t*)ad);
	ad+= sizeof(CFG[i]);
	ChekSum+= CFG[i];
	}

cks= eeprom_read_word((uint16_t*)ad);
if(ChekSum != cks) SetFactoryCFG();			//if first firing, parameter by default
}

//__________ Set factory configuration
void SetFactoryCFG(void)
{
int i;

for(i= 0; i < MAXPARAM; i++)
	{
	CFG[i]= ParamFactory[i];
	}

SaveCFG();
}
//______________________________ End parameters memory ______________________________



//__________ return wind speed in tenth of m/s
double WindSpeed(void)
{
double vmoy= 0;
int i;

//___filtering by running average 
for(i= 0; i < CFG[iFilteringAlarm]; i++)
	{
	vmoy+= Vitesses[i];
	}
vmoy/= CFG[iFilteringAlarm];
//___
return((int)(vmoy * CFG[iAnemoCalibrate] / 10.));
}

//__________ return wind angle in 
int WindAngle(void)
{
long dmoy= 0;
int i;

//___filtering by running average
for(i= 0; i < CFG[iFilteringAlarm]; i++)
	{
	dmoy+= (long)Directions[i];
	}
dmoy/= CFG[iFilteringAlarm];
//___
dmoy= (dmoy * 360) / (CFG[iAngleCalibrate] * 2);		//degrees conversion (*2 because coder in double resolution)
return((int)dmoy);
}

//__________ Buzzers control 
void Beep(unsigned int v)
{
if(v > 0)
	{
	TmpBeep= v;					//set length bip
	BUZZER= ON;					//enable
	}
else
	BUZZER= OFF;				//disable
}

//__________ Is true if beep active
bool BeepActive(void)
{
if(TmpBeep) return(true);

return(false);
}

//__________ Is true if marginal conditions
bool MarginalConditions(void)
{
int wspeed= (int)WindSpeed() / 10;

if((wspeed < CFG[iSetWindMin]) || (wspeed > CFG[iSetWindMax]) || (abs(WindAngle()) > CFG[iSetAngleMax]))
	return(true);

return(false);
}

//__________ Alarm mode 1 : xx continuous secondes of marginals condition
//averaging or not (for FAI= 20s without averaging), then continuous beep of xx secondes 
void AlarmModeFAI(void)
{
int static stal;
	
switch(stal)
	{
	case 0:
		if(MarginalConditions() == YES)
			{
			Beep(400);					//intermittent alarm beep
			CntCondition++ ;			//marginals condition counter in seconds
			}
		else
			{
			CntCondition= 0;
			}
		
		if(CntCondition >= CFG[iTimeAlarm])
			{
			Beep(CFG[iDurAlarm] * SECOND);			//continuous beep during  xx seconds
			stal= 1;
			}
	break;
	case 1:
		if(BeepActive() == NO)
			{
			stal= 2;
			}
	break;
	case 2:
		if(MarginalConditions() == NO)
			{
			stal= 0;
			}
		else
			{
			Beep(SECOND+100);			//continuous beep because overlapping beeps
			}
	break;
	default:
		stal= 0;
	}
}

//__________ Alarm mode 2 : xx accumulated seconds of marginals condition
//averaging or not, then continuous beep of xx secondes
int CntRaz= 0;
void AlarmModeCumul(void)
{
int static stal;

switch(stal)
	{
	case 0:
		if(MarginalConditions() == YES)
			{
			Beep(400);					//intermittent alarm beep
			CntRaz= 0;
			CntCondition++ ;			//marginals condition counter in seconds
			}
		else
			{
			if(CntRaz++ >= CFG[iResetCumul])
				{
				if(CntCondition > 0) Beep(40);				//beep auto reset cumul
				CntRaz= 0;
				CntCondition= 0;
				}
			}
	
		if(CntCondition >= CFG[iTimeAlarm])
			{
			Beep(CFG[iDurAlarm] * SECOND);			//continuous beep during  xx seconds
			CntCondition= 0;
			stal= 1;
			}
	break;
	case 1:
		if(BeepActive() == NO)
			{
			stal= 2;
			}
	break;
	case 2:
		if(MarginalConditions() == NO)
			{
			stal= 0;
			}
		else
			{
			Beep(SECOND+100);			//continuous beep because overlapping beeps
			}
	break;
	default:
		stal= 0;
	}
}


/*__________ Alarm marginal conditions process
bip discontinu (0.5 Hz) si conditions marginales instantannes (vitesse et angle), puis suivant mode, bip continu si :
mode 0 : alarm dsactive
mode 1 : xx secondes continues de conditions marginales moyennes ou non, puis bip continu de XX secondes,  (pour FAI= 20s sans moyennage)
mode 2 : xx secondes cumules de conditions marginales moyennes ou non, puis bip continu de XX secondes, raz auto du cumul au bout de xx secondes si alarme non dclenche
mode 3 : xx % de la dure du run en conditions marginales moyennes ou non, necessite une liaison de synchro  ChronoDS
*/
void AMC(void)
{
if(ReqAMC)		//treatment requested (see IT millisecond)
	{
	switch(CFG[iSetModeAlarm])
		{
		case 1:		// xx continuous secondes of marginals condition
			AlarmModeFAI();
		break;
		case 2:		//xx accumulated seconds of marginals condition 
			AlarmModeCumul();
		break;
		}
		
	ReqAMC= OFF;		//request acquitment
	}
}


//______________________________ SPI functions ______________________________

//___ Delay s
void Delayus(unsigned int t)
{
unsigned int j;

for(j= 0; j < t; j++)		//loop ~1s (for 328 at 16 MHz)
	{
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	}
}

//___SPI init
#define SCK		PB5
#define MOSI	PB3
#define SS		PB2

#define SCK_HIGH PORTB|= (1 << SCK)						//set SCK to high
#define SCK_LOW PORTB&= ~(1 << SCK)						//set SCK to low

#define MOSI_HIGH PORTB|= (1 << MOSI)					//set MOSI to high
#define MOSI_LOW PORTB&= ~(1 << MOSI)					//set MOSI to low

#define SS_HIGH PORTB|= (1 << SS)						//set SS to high
#define SS_LOW PORTB&= ~(1 << SS)						//set SS to low

//__________ Init SPI interface
void SPI_MasterInit(void)
{
DDRB|= (1 << MOSI) | (1 << SCK) | (1 << SS);			//set MOSI, SCK and SS as output
SS_HIGH;
SCK_HIGH;
}

//__________ SPI master transfer (make in code C because C SCK frequency is too hight for the display)
void SPI_MasterTransfer(char data)
{
int i;

SS_LOW;
for(i= 0; i < 8; i++)			//send data
	{
	Delayus(50);
	SCK_LOW;
	if(data & 0b10000000)
		{
		MOSI_HIGH;
		} 
	else
		{
		MOSI_LOW;
		}
	Delayus(50);
	SCK_HIGH;
	data= data << 1;
	}

Delayus(50);
SS_HIGH;
Delayus(110);
}

//__________ Sending string of characters with EOS (not used)
void SPI_StrSend(char *s)
{
LCDSPI_Clear();

SPI_MasterTransfer(0xFE);
SPI_MasterTransfer(0x53);						//Set backlight brightness
SPI_MasterTransfer(CFG[iBacklight] + 1);	//value of brightness

do
	{
	SPI_MasterTransfer(*s++);
	}
while(*(s-1));
}

//__________ Display string with her length specified
void LCDSPI_String(char *str, int lng)
{
char *p= str;

while(*p && (p - str) < lng)							//sendind string
	{
	if(*p >= ' ') SPI_MasterTransfer(*p);			//control codes filtering
	p++;
	}
}

//__________ Screen displaying
void Display_2x16(bool clear, char *s)
{
SPI_MasterTransfer(0xFE);
SPI_MasterTransfer(0x53);									//Set backlight brightness
SPI_MasterTransfer((char)(CFG[iBacklight] + 1));	//value of brightness (+1 because brightness param of display begin to 1)
if(clear) LCDSPI_Clear();

SPI_MasterTransfer(0xFE);
SPI_MasterTransfer(0x45);						//set cursor
SPI_MasterTransfer(0x00);						//in line 1

LCDSPI_String(s, LNG_LINE);					//sendind first line

SPI_MasterTransfer(0xFE);
SPI_MasterTransfer(0x45);						//set cursor
SPI_MasterTransfer(0x40);						//in line 2

LCDSPI_String(s + LNG_LINE, LNG_LINE);		//sendind second line
}

//__________ Clear Screen
void LCDSPI_Clear(void)
{
SPI_MasterTransfer(0xFE);
SPI_MasterTransfer(0x51);						//display clear, cursor in line 1
Delayus(2000);
}


//______________________________ IHM ______________________________
//__________ Displaying selected parameter
void DisplayCFG(int param, int val)
{
char str[50];
PGM_P pnt;

memcpy_P(&pnt, &StringCFG[param + MAXEDIT * CFG[iLanguage]], sizeof(PGM_P));
strcpy_P(str, pnt);
sprintf(Dialog, "%s > %d", str, val);
Display_2x16(ON, Dialog);

SPI_MasterTransfer(0xFE);
SPI_MasterTransfer(0x49);			//move cursor left
}

//__________ Limitation value
inline void Limit(int *v, int max, int min)
{
if(*v > max) 
	*v= max;
else
	if(*v < min) 
		*v= min;
}

//__________ Battery voltage treatment
double VbatTreatment(void)
{
double vb;

vb= REF_ADC * Vbat / RES_ADC;										//volt conversion
vb=  vb * CFG[iVbatCalibrate] / 1000.;							//scalling follow divisor bridge
if(vb < (double)(CFG[iBatThreshold] / 10.))					//threshold battery in tenth of volt
	{
	for(int i= 0; i < 3; i++)
		{
		Beep(40);
		TimerSet(100); while(TimerActive());
		}
	}
	
return(vb);
}

//__________ Config mode selection
void SetConfigMode(void)
{
ReqAMC= OFF;													//request acquitment
Mode= CONFIG;
PCMSK1= (1 << PCINT11);										//enable IT phase A IHM coder
IndParam= 0;													//first param selection
ShortClick= ON;
Beep(OFF);
IT_IHM= IHMCODER;												//for displaying immediately config screen
}

//__________ Measure mode selection
void SetMeasureMode(void)
{
Mode= MEASURE;
TimeBase= 0;
Angle= 0;
ShortClick= ON;
PCMSK1= (1 << PCINT9);										//enable IT phase A angle coder
IT_IHM= ANGCODER;												//for displaying immediately measure screen
}

//__________ IHM gestion
void IHM(void)
{
int tmpclick= 0;
		
if(IHM_CLICK == 0)			//click pressed
	{
	TimerSet(100); while(TimerActive());						//bounce filtering
	while((IHM_CLICK == 0) && (tmpclick < LONGKEY))			//long click measure
		{
		TimerSet(10); while(TimerActive());
		tmpclick++;
		}
	
	if(tmpclick >= LONGKEY)
		{
		Beep(100);
		while((IHM_CLICK == 0));								//waiting release
		if((Mode == MEASURE)) 									//mode changing : toggle between MEASURE and CONFIG
			{
			SetConfigMode();										//set config mode
			} 
		else
			{
			SaveCFG();												//parameters saving when quit config mode
			SetMeasureMode();										//set measure mode
			}
		}
	else
		{
		Beep(50);
		ShortClick= !ShortClick;
		}
	}
	
if(IT_IHM == ANGCODER)
	{
	double vb= VbatTreatment();									//battery voltage treatment
		
	if(ShortClick)
		{	//___displaying wind's measures
		sprintf(Dialog, "Anemo:%5.1f m/s Angle:%4d deg  ", WindSpeed() / 10., WindAngle());			//wind speed in tenth of m/s
		Display_2x16(OFF, Dialog);
		IT_IHM= FREE;
		}
	else
		{	//___displaying vbat...
		if(CFG[iLanguage] == 0)
			sprintf(Dialog, "Battery :%5.1f V-----------------", vb);
		else
			sprintf(Dialog, "Batterie :%4.1f V-----------------", vb);

		Display_2x16(OFF, Dialog);
		IT_IHM= FREE;
		}
	}
			
if(ShortClick && (Mode == CONFIG))
	{
	SPI_MasterTransfer(0xFE);
	SPI_MasterTransfer(0x47);			//show cursor
	}
else
	{
	SPI_MasterTransfer(0xFE);
	SPI_MasterTransfer(0x48);			//show cursor
	}
	
if(IT_IHM == IHMCODER)
	{
	
	param= pgm_read_word(&EditCFG[IndParam][0]);
	paramax= pgm_read_word(&EditCFG[IndParam][1]);
	paramin= pgm_read_word(&EditCFG[IndParam][2]);
	DisplayCFG(IndParam, CFG[param]);									//display actual param

	TimerSet(10); while(TimerActive());									//bounce filtering
	TimeBase= 0;
	IT_IHM= FREE;
	}
}
//______________________________ End IHM ______________________________


//__________ init and main loop of tasks
int main(void)
{
PRR= 0x00 | (1 << PRTWI) | (1 << PRUSART0) | (1 << PRTIM0);			//for power reduce

//__timer2 (8bit) : @16 MHz / 128 => run @ 125KHz = priode 8S => reload value for milliseconds Timer, tempo...
TCCR2B= (1 << CS20) | (1 << CS22);											//128 divide
TIMSK2|= (1 << TOIE2);															//overflow IT enable

//__IT codeurs angle, IHM
PCICR|= (1 << PCIE1);															//enable pin change interrupt PCINT8..15

//__timer1 (16 bits) : external clock source on T1 pin, clock on falling edge, for speed coder (anemo)
TCCR1B|= (1 << CS12) | (1 << CS11);

//__enable and init ADC (for battery voltage measurement)
ADCSRA= (1 << ADEN) | (1 << ADPS2) | (1 << ADPS1);						//ADEN: enable ADC, ADPS2: ADPS2 and ADPS0 set to make division factor 32
ADMUX= (1 << REFS0);																//voltage ref is AVCC, ADC0 input channel (PC0)

//__set outputs
BUZ_DIR= OUTPUT;
BUG_DIR= OUTPUT;

//__set pull-up on inputs
VIT_V= 1;											//PORTD5 : anmo input
ANG_VA= 1;											//PORTC1 : phase A coder angle PCINT9
ANG_VB= 1;											//PORTC2 : phase B coder angle

IHM_A= 1;											//PORTC3 : phase A coder IHM PCINT11
IHM_B= 1;											//PORTC4 : phase B coder IHM
IHM_CLICK= 1;										//PORTC5 : switch coder IHM PCINT13
//__
SPI_MasterInit();
LoadCFG();											//configuration loadind
sei();												//global it enable


TimerSet(300); while(TimerActive());
SPI_MasterTransfer(0xFE);
SPI_MasterTransfer(0x53);						//Set backlight brightness
SPI_MasterTransfer(4);							//because at fireing brightness if full

//__start screen
sprintf(Dialog, "-- %s --  DS DESIGN %s ", MODULE_NAME, VERSION);
Display_2x16(ON, Dialog);
TimerSet(1500); while(TimerActive());
SetConfigMode();
Beep(200);

while(1)
	{
	AMC();									//task AMC (alarm marginal conditions)
	IHM();									//task IHM (interface human / machine)
	}
}

//___ End of file
