/*						///  Module mto F3F  \\\

	Serge Delarbre 15.05.15

*/

#include <stdbool.h>

enum
{
FREE,
CLICK,
IHMCODER,
ANGCODER
};

enum
	{
	MEASURE,
	CONFIG
	};



#define F_CPU 16000000UL  //Hz
//#define IT_1ms 250						//recharge timer1 pour IT 1ms
#define IT_1ms F_CPU/128000						//recharge timer2 pour IT 1ms

#define SIZ_BUF_RX 100
#define MAXMOY		15				//taille max des tables de moyennage mesures

#define PROMPT "\r\n"
#define INPUT 0
#define OUTPUT 1

#define LONGKEY	100
#define PRESSED	0
#define RELEASED	1
#define TRUE 		1
#define FALSE 		0
#define ON 			1
#define OFF 		0
#define OK 			1
#define NO 			0
#define YES 		1
#define ERR 		0
#define CR 			13
#define EOS 		0					//end of string

#define REF_ADC 		5.0			//tension de rfrence en mV pour conversion analogique
#define RES_ADC		1024.0		//resolution du convertisseur analogique
#define kVBAT			1695			//coef pour mesure Vbat

#define SECOND 	1000				//tempo 1 seconde
#define T500MS 	500				//tempo demi-seconde

typedef struct
	{
	unsigned int b0:1;
	unsigned int b1:1;
	unsigned int b2:1;
	unsigned int b3:1;
	unsigned int b4:1;
	unsigned int b5:1;
	unsigned int b6:1;
	unsigned int b7:1;
	} _bitfield; 

//___ Macro REGBIT inspired by stu_san on AVR Freaks
#define REGBIT(reg,n)		((volatile _bitfield*)&reg)->b##n
#define SBI(ad, bit)			((ad) |= (uint8_t)(1 << bit))
#define CBI(ad, bit)			((ad) &= (uint8_t)~(1 << bit))

//___ I/Os definitions
#define VIT_V    				REGBIT(PIND, 5)		//vitesse vent : phase A ou B codeur anemo = entre clock timer1 (pin T1)

#define V_BAT					REGBIT(PINC, 0)		//tension alimentation : mesure analogique
#define ANG_VA    			REGBIT(PINC, 1)		//angle vent : phase A codeur angle PCINT9
#define ANG_VB    			REGBIT(PINC, 2)		//angle vent : phase B codeur angle

#define IHM_A    				REGBIT(PINC, 3)		//phase A codeur IHM PCINT11
#define IHM_B    				REGBIT(PINC, 4)		//phase B codeur IHM
#define IHM_CLICK				REGBIT(PINC, 5)		//switch codeur IHM PCINT13

#define BUZZER		 			REGBIT(PORTD,3)		//buzzer conditions marginales
#define BUZ_DIR				REGBIT(DDRD,3)	

#define BUGTRACE		 		REGBIT(PORTD,7)		//sortie pour trace debug
#define BUG_DIR				REGBIT(DDRD,7)	

//______________________________ IHM ______________________________
#define MAXEDIT 15		//nb param
#define MAXPARAM MAXEDIT
enum		//for direct access to parameters
{
	iSetZeroAngle,
	iSetAngleMax,
	iSetWindMin,
	iSetWindMax,
	iSetModeAlarm,
	iTimeAlarm,
	iDurAlarm,
	iResetCumul,
	iFilteringAlarm,
	iBatThreshold,
	iAngleCalibrate,
	iAnemoCalibrate,
	iVbatCalibrate,
	iLanguage,
	iBacklight
};

#define LNG_LINE 16
//							 English texts   	 ----------------================
const char aSetZeroAngle[] PROGMEM=		" SET ZERO ANGLE THEN LONG CLICK ";
const char aSetAngleMax[] PROGMEM=		" MAXIMUM ANGLE  DEGREE";
const char aSetWindMin[] PROGMEM=		"  MINIMUM WIND  m/s";
const char aSetWindMax[] PROGMEM=		"  MAXIMUM WIND  m/s";
const char aSetModeAlarm[] PROGMEM=		"   ALARM MODE   VALUE";
const char aSecondAlarm[] PROGMEM=		"  TIME BEFORE   ALARM SEC.";
const char aDurAlarm[] PROGMEM=			" DURATION ALARM SECOND";
const char aResetCumul[] PROGMEM=		"RESET CUMULATIVETIME SEC.";
const char aFilteringAlarm[] PROGMEM=	"  FILTERING OF  MEASURES";
const char aBatThreshold[] PROGMEM=		"BATTERY VOLTAGE THRESHOLD";
const char aAngleCalibrate[] PROGMEM=	"WARNING ! ANGLE CALIBRAT.";				//angle coder resolution
const char aAnemoCalibrate[] PROGMEM=	"WARNING ! ANEMO CALIBRAT.";				//wind speed measure calibration
const char aVbatCalibrate[] PROGMEM=	"WARNING ! VBat  CALIBRAT.";				//battery voltage measure measure calibration
const char aLanguage[] PROGMEM=			"  SET LANGUAGE  0=EN, 1=FR ";
const char aBacklight[] PROGMEM=			" SET BACKLIGHT  BRIGHTNESS";

//							 French texts   	 ----------------================
const char fSetZeroAngle[] PROGMEM=		"POS. ANGLE ZERO  PUIS CLIC LONG ";
const char fSetAngleMax[] PROGMEM=		" ANGLE MAXIMUM  DEGRE";
const char fSetWindMin[] PROGMEM=		"  VENT MINIMUM  m/s";
const char fSetWindMax[] PROGMEM=		"  VENT MAXIMUM  m/s";
const char fSetModeAlarm[] PROGMEM=		"  MODE ALARME   VALEUR";
const char fSecondAlarm[] PROGMEM=		"  TEMPS AVANT   ALARME";
const char fDurAlarm[] PROGMEM=			"  DUREE ALARME  SECONDE";
const char fResetCumul[] PROGMEM=		"RAZ TEMPS CUMULESECONDE";
const char fFilteringAlarm[] PROGMEM=	"  FILTRAGE DES  MESURES";
const char fBatThreshold[] PROGMEM=		" SEUIL TENSION  BATTERIE";
const char fAngleCalibrate[] PROGMEM=	"CALIB. ANGLE !!!VALEUR";					//angle coder resolution
const char fAnemoCalibrate[] PROGMEM=	"CALIB. ANEMO !!!VALEUR";					//wind speed measure calibration
const char fVbatCalibrate[] PROGMEM=	"CALIB. VBat !!! VALEUR";							//battery voltage measure measure calibration
const char fLanguage[] PROGMEM=			" CHOIX LANGAGE  0=EN, 1=FR ";
const char fBacklight[] PROGMEM=			"RETRO-ECLAIRAGE LUMINOSITE";



PGM_P const StringCFG[MAXEDIT * 2] PROGMEM=		//array of display's strings pointers
{
	aSetZeroAngle,
	aSetAngleMax,
	aSetWindMin,
	aSetWindMax,
	aSetModeAlarm,
	aSecondAlarm,
	aDurAlarm,
	aResetCumul,
	aFilteringAlarm,
	aBatThreshold,
	aAngleCalibrate,
	aAnemoCalibrate,
	aVbatCalibrate,
	aLanguage,
	aBacklight,
		
	fSetZeroAngle,
	fSetAngleMax,
	fSetWindMin,
	fSetWindMax,
	fSetModeAlarm,
	fSecondAlarm,
	fDurAlarm,
	fResetCumul,
	fFilteringAlarm,
	fBatThreshold,
	fAngleCalibrate,
	fAnemoCalibrate,
	fVbatCalibrate,
	fLanguage,
	fBacklight
};

const int EditCFG[MAXEDIT][3] PROGMEM= 				//parameters list
{
	{iSetZeroAngle, 1, 0},
	{iSetAngleMax, 90, 0},
	{iSetWindMin, 10, 0},
	{iSetWindMax, 99, 0},
	{iSetModeAlarm, 2, 0},
	{iTimeAlarm, 60, 0},
	{iDurAlarm, 20, 3},
	{iResetCumul, 90, 10},
	{iFilteringAlarm, MAXMOY, 1},
	{iBatThreshold, 100, 50},
	{iAngleCalibrate, 500, 0},
	{iAnemoCalibrate, 500, 0},
	{iVbatCalibrate, 3000, 0},
	{iLanguage, 1, 0},
	{iBacklight, 3, 0}
};

const int ParamFactory[]=									//default parameters value
{
	0,
	45,
	0,				//debug wind mini 0 m/s, normaly 3 m/s
	25,
	1,				//Mode alarm
	20,
	5,
	30,
	1,
	72,			//threshold for LiPo 2S, in tenth of volt
	100,			//anemo coder = 100 points for 360
	77,
	1695,
	0,				//english language
	0				//backlight off
};

//______________________________ End IHM ______________________________

//___ Proto

void TimerSet(int);
bool TimerActive(void);

void SaveCFG(void);
void LoadCFG(void);
void SetFactoryCFG(void);

void GetVbat(void);
void Beep(unsigned int);
bool BeepActive(void);

void SPI_MasterInit(void);
void SPI_MasterTransfer(char);
void SPI_StrSend(char*);
void Delayus(unsigned int);
void Display_2x16(bool, char*);
void LCDSPI_String(char*, int);
void DisplayCFG(int, int);
void LCDSPI_Clear(void);
void Limit(int*, int, int);
void IHM(void);
double WindSpeed(void);
int WindAngle(void);
void SetConfigMode(void);
void SetMeasureMode(void);
double VbatTreatment(void);
void Conditions(void);
void AlarmModeCumul(void);
void AlarmModeFAI(void);
bool MarginalConditions(void);

//___ End of file